import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

from scipy.stats import median_abs_deviation

print('\t\t\t\t\t Escrito por : Roberto \n')

def compute_gap(arr):
    # Funcion para calcular el gap
    median = np.median(arr, axis=0)
    MAD = median_abs_deviation(arr)
    gap = MAD/(2*median)
    return gap

def gini(arr):
        # Funcion para realizar el calculo de Gini
        count = arr.size
        coefficient = 2 / count
        indexes = np.arange(1, count + 1)
        weighted_sum = (indexes * arr).sum()
        total = arr.sum()
        constant = (count + 1) / count
        return coefficient * weighted_sum / total - constant

def lorenz(arr):
        # Aqui se divide la suma del prefij por la suma total
        # Esto asegura que todos los valores esten entre 0 y 1.
        scaled_prefix_sum = arr.cumsum() / arr.sum()
        return np.insert(scaled_prefix_sum, 0, 0)

def jfi(data):
    # Tenemos que sumA es la media de los valores elevado al cuadrado.
    sumA=data.sum()**2
    n=0
    for ele in data['datos']:
        n+=ele**2
    # Tenemos que sumB es el numero de datos por la suma de los cuadrados de cada dato.
    sumB=n*data['datos'].count()
    return sumA/sumB

try :
    choice = int(input("[1] PRESIONE 1 PARA ESTUDIAR SOLO UN ARCHIVO \n[2] PRESIONE 2 PARA USAR 2 ARCHIVOS \n ELIGE TU OPCION 1 O 2 : "))
    text_file = open("data.txt", "r")
    lines = text_file.read().split(',') 
    text_file.close()
    input_list = list(map(int, lines))
    input_list.sort()
    arr = np.array(input_list)
    data={'datos': arr}
    dataframe=pd.DataFrame(data=data)

    if choice==1:
        print('\n\n Se haran los calculos sobre los siguientes datos: ',input_list)
        
        # Mostramos el coeficiente de Gini, Jain y Gap
        print("Coeficiente de Gini = ",gini(arr))
        print("JFI = ", jfi(dataframe))
        print("GAP = ", compute_gap(arr))
        lorenz_curve = lorenz(arr)

        # El valor siempre tiene que estar entre 0 y 1.
        plt.plot(np.linspace(0.0, 1.0, lorenz_curve.size), lorenz_curve)
        # Dibujamos la linea de equidad perfecta de la curva.
        plt.plot([0,1], [0,1])
        plt.show()
    if choice==2:
        text_file2 = open("data2.txt", "r")
        lines2 = text_file2.read().split(',') 
        text_file2.close()
        input_list2 = list(map(int, lines2))
        input_list2.sort()
        print("Gini Index of Data 1 file = ",gini(arr))
        print("JFI Datos 1 = ", jfi(dataframe))
        print("GAP Datos 1 = ", compute_gap(arr))

        lorenz_curve = lorenz(arr)

        arr2 = np.array(input_list2)
        data2={'datos': arr2}
        dataframe2=pd.DataFrame(data=data2)

        print("Gini Index Datos 2 = ",gini(arr2))
        print("JFI Datos 2 = ", jfi(dataframe2))
        print("GAP Datos 2 = ", compute_gap(arr2))

        lorenz_curve2 = lorenz(arr2)
        # Dibujamos la linea de equidad perfecta de la curva.
        plt.plot([0,1], [0,1], color="black")

        # El valor siempre tiene que estar entre 0 y 1.
        plt.plot(np.linspace(0.0, 1.0, lorenz_curve.size), lorenz_curve, color="red", label="Prueba Remoto Grupos de Usuarios")
        plt.plot(np.linspace(0.0, 1.0, lorenz_curve2.size), lorenz_curve2, label="Prueba Remoto")
        plt.legend(loc="upper left")
        plt.show()
    
    

    
except Exception as e:
    print(e)
    print("\n\nNO SE HA ENCONTRADO NINGUN ARCHIVO, GUARDALO COMO DATA.TXT Y VUELVE A EJECUTAR EL PROGRAMA DE NUEVO")
    print("\nLOS DATOS DEBEN ESTAR GUARDADOS UNICAMENTE CON , COMO EN EL EJEMPLO SIGUIENTE: '30,29,40,32'")
    input("\n\nPRESIONA CUALQUIER BOTON PARA SALIR...")
